﻿/********************************** Module Header **********************************\
Module Name:  FileContextMenuExt.cs
Project:      CSShellExtContextMenuHandler
Copyright (c) Microsoft Corporation.

 * Modified by Jan Hannemann - Coronox
 * 
 * 
The FileContextMenuExt.cs file defines a context menu handler by implementing the 
IShellExtInit and IContextMenu interfaces.

This source is subject to the Microsoft Public License.
See http://www.microsoft.com/opensource/licenses.mspx#Ms-PL.
All other rights reserved.

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER 
EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
\***********************************************************************************/

#region Using directives
using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;
using Coronox.SkyShellEx.Properties;
using System.Drawing;
using System.Collections.Specialized;
using Microsoft.Win32;
using System.Diagnostics;
using System.IO;
using Coronox.SkyShell;

#endregion


namespace Coronox.SkyShellEx
{
    [ClassInterface(ClassInterfaceType.None)]
    [Guid("F3BF3833-BC9C-411D-804E-A3C40D67BA55"), ComVisible(true)]
    public class FileContextMenuExt : IShellExtInit, IContextMenu
    {
        // The name of the selected file.
        //private string selectedFile;
        //private string _skydriveFolder;

        public StringCollection _selectedFiles;

        // private string _selectedFolder;
        public string SelectedFolder
        {
            get
            {
                return _selectedFiles != null && _selectedFiles.Count > 0 ? _selectedFiles[0] : string.Empty;
            }
            set
            {
                // _selectedFolder = value;

            }
        }


        //public bool IsSyncedFolder
        //{
        //    get
        //    {
        //        if (!string.IsNullOrEmpty(SelectedFolder) && Directory.Exists(SelectedFolder))
        //        {
        //            if (SelectedFolder.ToLower().StartsWith(GetSkyDriveFolder().ToLower()))
        //            {
        //                return true;
        //            }
        //            else
        //                return false;
        //        }
        //        else
        //        {
        //            return false;
        //        }
        //    }
        //}

        //public bool IsSyncedJunctionFolder
        //{
        //    get
        //    {
        //        if (IsSyncedFolder)
        //        {
        //            return CheckForJunction(SelectedFolder);
        //        }
        //        else
        //            return false;
        //    }
        //}


        private string _syncMenuText = "&Sync to SkyDrive";
        private string _stopSyncMenuText = "&Stop Syncing to SkyDrive";
        private string _sharecontentMenuText = "&Share from SkyDrive";

        private IntPtr menuBmp = IntPtr.Zero;
        private IntPtr _stopSyncBmp = IntPtr.Zero;
        private IntPtr _syncBmp = IntPtr.Zero;
        private IntPtr _shareBmp = IntPtr.Zero;


        private string _syncVerb = "csdisplay";
        private string _syncCanonicalName = "CSDisplayFileName";
        private string _syncHelpText = "Sync folder";


        private string _stopSyncVerb = "stopsync";
        private string _stopSyncCanonicalName = "SkydriveStopSync";
        private string _stopSyncHelpText = "Stops syncing a folder to Skydrive";

        private string _sharecontentVerb = "sharecontent";
        private string _sharecontentCanonicalName = "SkydriveShareContent";
        private string _sharecontentHelpText = "Shares a folder from Skydrive";


        private uint IDM_DISPLAY = 0;
        private uint IDM_DISPLAY2 = 1;
        private uint IDM_DISPLAY3 = 2;

        public FileContextMenuExt()
        {
            // Load the bitmap for the menu item.
            Bitmap bmp = Coronox.SkyShellEx.Properties.Resources.skydrive;
            //bmp.MakeTransparent(bmp.GetPixel(0, 0));
            this.menuBmp = bmp.GetHbitmap();

            Bitmap stopBmp = Coronox.SkyShellEx.Properties.Resources.StopSyncShelEx2;
            stopBmp.MakeTransparent(Color.FromArgb(255, 0, 255));
            this._stopSyncBmp = stopBmp.GetHbitmap();

            Bitmap sb = Coronox.SkyShellEx.Properties.Resources.SyncShelEx;
            sb.MakeTransparent(Color.FromArgb(255, 0, 255));
            this._syncBmp = sb.GetHbitmap();

            Bitmap sb2 = Coronox.SkyShellEx.Properties.Resources.ShareShelEx;
            sb2.MakeTransparent(Color.FromArgb(255, 0, 255));
            this._shareBmp = sb2.GetHbitmap();
        }

        ~FileContextMenuExt()
        {
            if (this.menuBmp != IntPtr.Zero)
            {
                NativeMethods.DeleteObject(this.menuBmp);
                NativeMethods.DeleteObject(this._stopSyncBmp);

                this.menuBmp = IntPtr.Zero;
                _stopSyncBmp = IntPtr.Zero;
            }
        }


        //public void CreateFolderJunction(string item, string junctionfolder)
        //{
        //    if (Directory.Exists(item))
        //    {
        //        DirectoryInfo di = new DirectoryInfo(item);
        //        string junctionPath = Path.Combine(junctionfolder, di.Name);
        //        Process p = new Process();
        //        ProcessStartInfo psi = new ProcessStartInfo();

        //        psi.FileName = "cmd.exe";
        //        psi.UseShellExecute = false;
        //        psi.CreateNoWindow = true;
        //        psi.WindowStyle = ProcessWindowStyle.Hidden;
        //        psi.Arguments = " /C mklink /J \"" + junctionPath + "\" \"" + di.FullName + "\"";
        //        // psi.RedirectStandardOutput = true;
        //        //  psi.RedirectStandardError = true;
        //        p.StartInfo = psi;

        //        //Console.WriteLine("{0} {1}", psi.FileName, psi.Arguments);

        //        p.Start();

        //        p.WaitForExit();
        //    }
        //}


        //internal void DeleteFolderJunction(string folder)
        //{
        //    if (Directory.Exists(folder))
        //    {


        //        Process p = new Process();
        //        ProcessStartInfo psi = new ProcessStartInfo();

        //        psi.FileName = "cmd.exe";
        //        psi.UseShellExecute = false;
        //        psi.CreateNoWindow = true;
        //        psi.WindowStyle = ProcessWindowStyle.Hidden;
        //        psi.Arguments = " /C rd \"" + folder + "\"";
        //        // psi.RedirectStandardOutput = true;
        //        //  psi.RedirectStandardError = true;
        //        p.StartInfo = psi;

        //        //Console.WriteLine("{0} {1}", psi.FileName, psi.Arguments);

        //        p.Start();

        //        p.WaitForExit();
        //    }
        //}

        internal string GetSkyDriveFolder()
        {
            var subkey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\SkyDrive");
            return subkey.GetValue("UserFolder").ToString();

        }

        internal bool CheckForJunction(string path)
        {
            DirectoryInfo di = new DirectoryInfo(path);
            bool ret = (di.Attributes & FileAttributes.ReparsePoint) == FileAttributes.ReparsePoint;
            return ret;

        }

        #region Shell Extension Registration

        [ComRegisterFunction()]
        public static void Register(Type t)
        {
            try
            {
                ShellExtReg.RegisterFolderShellExtContextMenuHandler(t.GUID,
                    "SkydriveSync");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message); // Log the error
                throw;  // Re-throw the exception
            }
        }

        [ComUnregisterFunction()]
        public static void Unregister(Type t)
        {
            try
            {
                ShellExtReg.UnregisterFolderShellExtContextMenuHandler(t.GUID, "SkydriveSync");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message); // Log the error
                throw;  // Re-throw the exception
            }
        }

        #endregion


        #region IShellExtInit Members

        /// <summary>
        /// Initialize the context menu handler.
        /// </summary>
        /// <param name="pidlFolder">
        /// A pointer to an ITEMIDLIST structure that uniquely identifies a folder.
        /// </param>
        /// <param name="pDataObj">
        /// A pointer to an IDataObject interface object that can be used to retrieve 
        /// the objects being acted upon.
        /// </param>
        /// <param name="hKeyProgID">
        /// The registry key for the file object or folder type.
        /// </param>
        public void Initialize(IntPtr pidlFolder, IntPtr pDataObj, IntPtr hKeyProgID)
        {
            //System.Windows.Forms.MessageBox.Show("Init");
            if (pDataObj == IntPtr.Zero)
            {
                throw new ArgumentException();
            }


            FORMATETC fe = new FORMATETC();
            fe.cfFormat = (short)CLIPFORMAT.CF_HDROP;
            fe.ptd = IntPtr.Zero;
            fe.dwAspect = DVASPECT.DVASPECT_CONTENT;
            fe.lindex = -1;
            fe.tymed = TYMED.TYMED_HGLOBAL;
            STGMEDIUM stm = new STGMEDIUM();

            // The pDataObj pointer contains the objects being acted upon. In this 
            // example, we get an HDROP handle for enumerating the selected files 
            // and folders.
            IDataObject dataObject = (IDataObject)Marshal.GetObjectForIUnknown(pDataObj);
            dataObject.GetData(ref fe, out stm);

            try
            {
                // Get an HDROP handle.
                IntPtr hDrop = stm.unionmember;
                if (hDrop == IntPtr.Zero)
                {
                    throw new ArgumentException();
                }

                _selectedFiles = new StringCollection();
                // Determine how many files are involved in this operation.
                uint nFiles = NativeMethods.DragQueryFile(hDrop, UInt32.MaxValue, null, 0);

                // This code sample displays the custom context menu item when only 
                // one file is selected. 
                if (nFiles == 1)
                {
                    // Get the path of the file.
                    StringBuilder fileName = new StringBuilder(260);
                    if (0 == NativeMethods.DragQueryFile(hDrop, 0, fileName,
                        fileName.Capacity))
                    {
                        Marshal.ThrowExceptionForHR(WinError.E_FAIL);
                    }
                    _selectedFiles.Add(fileName.ToString());
                    //this.selectedFile = fileName.ToString();
                }
                else
                {
                    Marshal.ThrowExceptionForHR(WinError.E_FAIL);
                }

                // [-or-]

                // Enumerate the selected files and folders.
                if (nFiles > 0)
                {

                    StringBuilder fileName = new StringBuilder(260);


                    for (uint i = 0; i < nFiles; i++)
                    {
                        // Get the next file name.
                        if (0 != NativeMethods.DragQueryFile(hDrop, i, fileName,
                            fileName.Capacity))
                        {
                            // Add the file name to the list.
                            _selectedFiles.Add(fileName.ToString());
                        }
                    }

                    // If we did not find any files we can work with, throw 
                    // exception.
                    if (_selectedFiles.Count == 0)
                    {
                        Marshal.ThrowExceptionForHR(WinError.E_FAIL);
                    }
                }
                else
                {
                    Marshal.ThrowExceptionForHR(WinError.E_FAIL);
                }
            }
            finally
            {
                NativeMethods.ReleaseStgMedium(ref stm);
            }
        }

        #endregion





        #region IContextMenu Members

        /// <summary>
        /// Add commands to a shortcut menu.
        /// </summary>
        /// <param name="hMenu">A handle to the shortcut menu.</param>
        /// <param name="iMenu">
        /// The zero-based position at which to insert the first new menu item.
        /// </param>
        /// <param name="idCmdFirst">
        /// The minimum value that the handler can specify for a menu item ID.
        /// </param>
        /// <param name="idCmdLast">
        /// The maximum value that the handler can specify for a menu item ID.
        /// </param>
        /// <param name="uFlags">
        /// Optional flags that specify how the shortcut menu can be changed.
        /// </param>
        /// <returns>
        /// If successful, returns an HRESULT value that has its severity value set 
        /// to SEVERITY_SUCCESS and its code value set to the offset of the largest 
        /// command identifier that was assigned, plus one.
        /// </returns>
        public int QueryContextMenu(
            IntPtr hMenu,
            uint iMenu,
            uint idCmdFirst,
            uint idCmdLast,
            uint uFlags)
        {
            //System.Windows.Forms.MessageBox.Show("1");
            // If uFlags include CMF_DEFAULTONLY then we should not do anything.
            if (((uint)CMF.CMF_DEFAULTONLY & uFlags) != 0)
            {
                return WinError.MAKE_HRESULT(WinError.SEVERITY_SUCCESS, 0, 0);
            }

            uint LastCommandId = 0;

            string reason;
            uint iMenuOffset = 0;
            if (Sky.CanSyncFolder(SelectedFolder, out reason, false) || Sky.CanStopSyncFolder(SelectedFolder, out reason) || Sky.CanShareFolder(SelectedFolder, out reason)) //lets whether we have to do anything 
            {
                //insert the top seperator
                MENUITEMINFO sepTop = new MENUITEMINFO();
                sepTop.cbSize = (uint)Marshal.SizeOf(sepTop);
                sepTop.fMask = MIIM.MIIM_TYPE;
                sepTop.fType = MFT.MFT_SEPARATOR;
                // sepTop.wID = iMenu + iMenuOffset;
                if (!NativeMethods.InsertMenuItem(hMenu, iMenu + iMenuOffset, true, ref sepTop))
                {
                    return Marshal.GetHRForLastWin32Error();
                }



                if (Sky.CanSyncFolder(SelectedFolder, out reason, false))
                {
                    iMenuOffset++;

                    // Use either InsertMenu or InsertMenuItem to add menu items.
                    MENUITEMINFO mii = new MENUITEMINFO();
                    mii.cbSize = (uint)Marshal.SizeOf(mii);
                    mii.fMask = MIIM.MIIM_BITMAP | MIIM.MIIM_STRING | MIIM.MIIM_FTYPE |
                        MIIM.MIIM_ID | MIIM.MIIM_STATE;
                    mii.wID = idCmdFirst + IDM_DISPLAY;
                    mii.fType = MFT.MFT_STRING;
                    mii.dwTypeData = this._syncMenuText;
                    mii.fState = MFS.MFS_ENABLED;
                    mii.hbmpItem = this._syncBmp;
                    if (!NativeMethods.InsertMenuItem(hMenu, iMenu + iMenuOffset, true, ref mii))
                    {
                        return Marshal.GetHRForLastWin32Error();
                    }

                    // Add a separator.
                    //MENUITEMINFO sep = new MENUITEMINFO();
                    //sep.cbSize = (uint)Marshal.SizeOf(sep);
                    //sep.fMask = MIIM.MIIM_TYPE;
                    //sep.fType = MFT.MFT_SEPARATOR;
                    //if (!NativeMethods.InsertMenuItem(hMenu, iMenu + 1, true, ref sep))
                    //{
                    //    return Marshal.GetHRForLastWin32Error();
                    //}

                    // Return an HRESULT value with the severity set to SEVERITY_SUCCESS. 
                    // Set the code value to the offset of the largest command identifier 
                    // that was assigned, plus one (1).
                    LastCommandId = IDM_DISPLAY2 + 1;
                    //return WinError.MAKE_HRESULT(WinError.SEVERITY_SUCCESS, 0,
                    //    IDM_DISPLAY + 2);
                }

                if (Sky.CanStopSyncFolder(SelectedFolder, out reason))
                {
                    iMenuOffset++;
                    MENUITEMINFO mii2 = new MENUITEMINFO();
                    mii2.cbSize = (uint)Marshal.SizeOf(mii2);
                    mii2.fMask = MIIM.MIIM_BITMAP | MIIM.MIIM_STRING | MIIM.MIIM_FTYPE |
                        MIIM.MIIM_ID | MIIM.MIIM_STATE;
                    mii2.wID = idCmdFirst + IDM_DISPLAY2;
                    mii2.fType = MFT.MFT_STRING;
                    mii2.dwTypeData = this._stopSyncMenuText;
                    mii2.fState = MFS.MFS_ENABLED;
                    mii2.hbmpItem = this._stopSyncBmp;
                    if (!NativeMethods.InsertMenuItem(hMenu, iMenu + iMenuOffset, true, ref mii2))
                    {
                        return Marshal.GetHRForLastWin32Error();
                    }

                    //// Add a separator.
                    //MENUITEMINFO sep = new MENUITEMINFO();
                    //sep.cbSize = (uint)Marshal.SizeOf(sep);
                    //sep.fMask = MIIM.MIIM_TYPE;
                    //sep.fType = MFT.MFT_SEPARATOR;
                    //if (!NativeMethods.InsertMenuItem(hMenu, iMenu + 1, true, ref sep))
                    //{
                    //    return Marshal.GetHRForLastWin32Error();
                    //}

                    // Return an HRESULT value with the severity set to SEVERITY_SUCCESS. 
                    // Set the code value to the offset of the largest command identifier 
                    // that was assigned, plus one (1).
                    LastCommandId = IDM_DISPLAY2 + 1;

                    //return WinError.MAKE_HRESULT(WinError.SEVERITY_SUCCESS, 0,
                    //    IDM_DISPLAY + 2);
                }

                if (Sky.CanShareFolder(SelectedFolder, out reason))
                {
                    iMenuOffset++;
                    MENUITEMINFO mii3 = new MENUITEMINFO();
                    mii3.cbSize = (uint)Marshal.SizeOf(mii3);
                    mii3.fMask = MIIM.MIIM_BITMAP | MIIM.MIIM_STRING | MIIM.MIIM_FTYPE |
                        MIIM.MIIM_ID | MIIM.MIIM_STATE;
                    mii3.wID = idCmdFirst + IDM_DISPLAY3;
                    mii3.fType = MFT.MFT_STRING;
                    mii3.dwTypeData = this._sharecontentMenuText;
                    mii3.fState = MFS.MFS_ENABLED;
                    mii3.hbmpItem = this._shareBmp;
                    if (!NativeMethods.InsertMenuItem(hMenu, iMenu + iMenuOffset, true, ref mii3))
                    {
                        return Marshal.GetHRForLastWin32Error();
                    }




                    // Return an HRESULT value with the severity set to SEVERITY_SUCCESS. 
                    // Set the code value to the offset of the largest command identifier 
                    // that was assigned, plus one (1).

                    LastCommandId = IDM_DISPLAY3 + 1;

                    //return WinError.MAKE_HRESULT(WinError.SEVERITY_SUCCESS, 0,
                    //    IDM_DISPLAY3 + 1);
                }



                //insert the bottom seperator
                iMenuOffset++;
                MENUITEMINFO sepBottom = new MENUITEMINFO();
                sepBottom.cbSize = (uint)Marshal.SizeOf(sepBottom);
                sepBottom.fMask = MIIM.MIIM_TYPE;
                sepBottom.fType = MFT.MFT_SEPARATOR;
                //  sepBottom.wID = iMenu + iMenuOffset;
                if (!NativeMethods.InsertMenuItem(hMenu, iMenu + iMenuOffset, true, ref sepBottom))
                {
                    return Marshal.GetHRForLastWin32Error();
                }




            }

            //If successful, returns an HRESULT value that has its severity value set to SEVERITY_SUCCESS and its code value set to the offset of the largest command identifier that was assigned, plus one.
            // For example, if idCmdFirst is set to 5 and you add three items to the menu with command identifiers of 5, 7, and 8, the return value should be MAKE_HRESULT(SEVERITY_SUCCESS, 0, 8 - 5 + 1).
            //Otherwise, it returns a COM error value.
            if (LastCommandId > 0)
                return WinError.MAKE_HRESULT(WinError.SEVERITY_SUCCESS, 0, LastCommandId);
            else
                return WinError.MAKE_HRESULT(WinError.SEVERITY_SUCCESS, 0, 0);
        }

        /// <summary>
        /// Carry out the command associated with a shortcut menu item.
        /// </summary>
        /// <param name="pici">
        /// A pointer to a CMINVOKECOMMANDINFO or CMINVOKECOMMANDINFOEX structure 
        /// containing information about the command. 
        /// </param>
        public void InvokeCommand(IntPtr pici)
        {
            bool isUnicode = false;



            // Determine which structure is being passed in, CMINVOKECOMMANDINFO or 
            // CMINVOKECOMMANDINFOEX based on the cbSize member of lpcmi. Although 
            // the lpcmi parameter is declared in Shlobj.h as a CMINVOKECOMMANDINFO 
            // structure, in practice it often points to a CMINVOKECOMMANDINFOEX 
            // structure. This struct is an extended version of CMINVOKECOMMANDINFO 
            // and has additional members that allow Unicode strings to be passed.
            CMINVOKECOMMANDINFO ici = (CMINVOKECOMMANDINFO)Marshal.PtrToStructure(
                pici, typeof(CMINVOKECOMMANDINFO));
            CMINVOKECOMMANDINFOEX iciex = new CMINVOKECOMMANDINFOEX();
            if (ici.cbSize == Marshal.SizeOf(typeof(CMINVOKECOMMANDINFOEX)))
            {
                if ((ici.fMask & CMIC.CMIC_MASK_UNICODE) != 0)
                {
                    isUnicode = true;
                    iciex = (CMINVOKECOMMANDINFOEX)Marshal.PtrToStructure(pici,
                        typeof(CMINVOKECOMMANDINFOEX));
                }
            }

            // Determines whether the command is identified by its offset or verb.
            // There are two ways to identify commands:
            // 
            //   1) The command's verb string 
            //   2) The command's identifier offset
            // 
            // If the high-order word of lpcmi->lpVerb (for the ANSI case) or 
            // lpcmi->lpVerbW (for the Unicode case) is nonzero, lpVerb or lpVerbW 
            // holds a verb string. If the high-order word is zero, the command 
            // offset is in the low-order word of lpcmi->lpVerb.

            // For the ANSI case, if the high-order word is not zero, the command's 
            // verb string is in lpcmi->lpVerb. 

            // System.Windows.Forms.MessageBox.Show(ici.verb.ToString());
            if (!isUnicode && NativeMethods.HighWord(ici.verb.ToInt32()) != 0)
            {

                System.Windows.Forms.MessageBox.Show(Marshal.PtrToStringAnsi(ici.verb));
                //System.Windows.Forms.MessageBox.Show("a");
                // Is the verb supported by this context menu extension?
                if (Marshal.PtrToStringAnsi(ici.verb) == this._syncVerb)
                {
                    
                    Sky.SyncFolder(SelectedFolder);
                    // CreateFolderJunction(SelectedFolder, GetSkyDriveFolder());
                    // System.Windows.Forms.MessageBox.Show("1");
                    //OnVerbDisplayFileName(ici.hwnd);
                }
                else if (Marshal.PtrToStringAnsi(ici.verb) == this._stopSyncVerb)
                {
                    Sky.StopSyncFolder(SelectedFolder, StopSyncOption.UserChoice);
                    //DeleteFolderJunction(SelectedFolder);
                    //System.Windows.Forms.MessageBox.Show("2");
                }
                else if (Marshal.PtrToStringAnsi(ici.verb) == this._sharecontentVerb)
                {
                    Sky.ShareFolder(SelectedFolder);
                }
                else
                {
                    // If the verb is not recognized by the context menu handler, it 
                    // must return E_FAIL to allow it to be passed on to the other 
                    // context menu handlers that might implement that verb.
                    Marshal.ThrowExceptionForHR(WinError.E_FAIL);
                }
            }

            // For the Unicode case, if the high-order word is not zero, the 
            // command's verb string is in lpcmi->lpVerbW. 
            else if (isUnicode && NativeMethods.HighWord(iciex.verbW.ToInt32()) != 0)
            {
                //  System.Windows.Forms.MessageBox.Show("b");
                // Is the verb supported by this context menu extension?
                if (Marshal.PtrToStringUni(iciex.verbW) == this._syncVerb)
                {
                    Sky.SyncFolder(SelectedFolder);
                    //System.Windows.Forms.MessageBox.Show("1");
                    //OnVerbDisplayFileName(ici.hwnd);
                }
                else if (Marshal.PtrToStringUni(iciex.verbW) == this._stopSyncVerb)
                {
                    Sky.StopSyncFolder(SelectedFolder, StopSyncOption.UserChoice);
                    //DeleteFolderJunction(SelectedFolder);
                    //  System.Windows.Forms.MessageBox.Show("2");
                }
                else if (Marshal.PtrToStringUni(iciex.verbW) == this._sharecontentVerb)
                {
                    Sky.ShareFolder(SelectedFolder);
                    //DeleteFolderJunction(SelectedFolder);
                    //  System.Windows.Forms.MessageBox.Show("2");
                }

                else
                {
                    // If the verb is not recognized by the context menu handler, it 
                    // must return E_FAIL to allow it to be passed on to the other 
                    // context menu handlers that might implement that verb.
                    Marshal.ThrowExceptionForHR(WinError.E_FAIL);
                }
            }

            // If the command cannot be identified through the verb string, then 
            // check the identifier offset.
            else
            {
                //  System.Windows.Forms.MessageBox.Show("c");
                // Is the command identifier offset supported by this context menu 
                // extension?
                if (NativeMethods.LowWord(ici.verb.ToInt32()) == IDM_DISPLAY)
                {
                    //System.Windows.Forms.MessageBox.Show("1");

                    Sky.SyncFolder(SelectedFolder);
                    // CreateFolderJunction(SelectedFolder, GetSkyDriveFolder());
                    //OnVerbDisplayFileName(ici.hwnd);
                }
                else if (NativeMethods.LowWord(ici.verb.ToInt32()) == IDM_DISPLAY2)
                {
                    //System.Windows.Forms.MessageBox.Show("2");
                    //System.Windows.Forms.MessageBox.Show(SelectedFolder);
                    Sky.StopSyncFolder(SelectedFolder, StopSyncOption.UserChoice);
                    //DeleteFolderJunction(SelectedFolder);

                }
                else if (NativeMethods.LowWord(ici.verb.ToInt32()) == IDM_DISPLAY3)
                {
                    //System.Windows.Forms.MessageBox.Show("2");
                    //System.Windows.Forms.MessageBox.Show(SelectedFolder);
                    Sky.ShareFolder(SelectedFolder);
                    //DeleteFolderJunction(SelectedFolder);

                }
                else
                {
                    // If the verb is not recognized by the context menu handler, it 
                    // must return E_FAIL to allow it to be passed on to the other 
                    // context menu handlers that might implement that verb.
                    Marshal.ThrowExceptionForHR(WinError.E_FAIL);
                }
            }
        }

        /// <summary>
        /// Get information about a shortcut menu command, including the help string 
        /// and the language-independent, or canonical, name for the command.
        /// </summary>
        /// <param name="idCmd">Menu command identifier offset.</param>
        /// <param name="uFlags">
        /// Flags specifying the information to return. This parameter can have one 
        /// of the following values: GCS_HELPTEXTA, GCS_HELPTEXTW, GCS_VALIDATEA, 
        /// GCS_VALIDATEW, GCS_VERBA, GCS_VERBW.
        /// </param>
        /// <param name="pReserved">Reserved. Must be IntPtr.Zero</param>
        /// <param name="pszName">
        /// The address of the buffer to receive the null-terminated string being 
        /// retrieved.
        /// </param>
        /// <param name="cchMax">
        /// Size of the buffer, in characters, to receive the null-terminated string.
        /// </param>
        public void GetCommandString(
            UIntPtr idCmd,
            uint uFlags,
            IntPtr pReserved,
            StringBuilder pszName,
            uint cchMax)
        {
            if (idCmd.ToUInt32() == IDM_DISPLAY)
            {
                switch ((GCS)uFlags)
                {
                    case GCS.GCS_VERBW:
                        if (this._syncCanonicalName.Length > cchMax - 1)
                        {
                            Marshal.ThrowExceptionForHR(WinError.STRSAFE_E_INSUFFICIENT_BUFFER);
                        }
                        else
                        {
                            pszName.Clear();
                            pszName.Append(this._syncCanonicalName);
                        }
                        break;

                    case GCS.GCS_HELPTEXTW:
                        if (this._syncHelpText.Length > cchMax - 1)
                        {
                            Marshal.ThrowExceptionForHR(WinError.STRSAFE_E_INSUFFICIENT_BUFFER);
                        }
                        else
                        {
                            pszName.Clear();
                            pszName.Append(this._syncHelpText);
                        }
                        break;
                }
            }

            if (idCmd.ToUInt32() == IDM_DISPLAY2)
            {
                switch ((GCS)uFlags)
                {
                    case GCS.GCS_VERBW:
                        if (this._syncCanonicalName.Length > cchMax - 1)
                        {
                            Marshal.ThrowExceptionForHR(WinError.STRSAFE_E_INSUFFICIENT_BUFFER);
                        }
                        else
                        {
                            pszName.Clear();
                            pszName.Append(this._stopSyncCanonicalName);
                        }
                        break;

                    case GCS.GCS_HELPTEXTW:
                        if (this._syncHelpText.Length > cchMax - 1)
                        {
                            Marshal.ThrowExceptionForHR(WinError.STRSAFE_E_INSUFFICIENT_BUFFER);
                        }
                        else
                        {
                            pszName.Clear();
                            pszName.Append(this._stopSyncHelpText);
                        }
                        break;
                }
            }

            if (idCmd.ToUInt32() == IDM_DISPLAY3)
            {
                switch ((GCS)uFlags)
                {
                    case GCS.GCS_VERBW:
                        if (this._sharecontentCanonicalName.Length > cchMax - 1)
                        {
                            Marshal.ThrowExceptionForHR(WinError.STRSAFE_E_INSUFFICIENT_BUFFER);
                        }
                        else
                        {
                            pszName.Clear();
                            pszName.Append(this._sharecontentCanonicalName);
                        }
                        break;

                    case GCS.GCS_HELPTEXTW:
                        if (this._sharecontentHelpText.Length > cchMax - 1)
                        {
                            Marshal.ThrowExceptionForHR(WinError.STRSAFE_E_INSUFFICIENT_BUFFER);
                        }
                        else
                        {
                            pszName.Clear();
                            pszName.Append(this._sharecontentHelpText);
                        }
                        break;
                }
            }

        }

        #endregion
    }
}